@@ -83,6 +83,7 @@ def self_test_suite():
     names = [
         'blackbox',
         'client',
+        'diff_tree',
         'fastexport',
         'file',
         'index',
@@ -103,10 +104,10 @@ def self_test_suite():
 
 def tutorial_test_suite():
     tutorial = [
-        '0-introduction',
-        '1-repo',
-        '2-object-store',
-        '3-conclusion',
+        'introduction',
+        'repo',
+        'object-store',
+        'conclusion',
         ]
     tutorial_files = ["../../docs/tutorial/%s.txt" % name for name in tutorial]
     def setup(test):
