@@ -304,6 +304,45 @@ class TestPack(PackTests):
                           commit.author)
         self.assertEquals([], commit.parents)
 
+    def _copy_pack(self, origpack):
+        basename = os.path.join(self.tempdir, 'somepack')
+        write_pack(basename, [(x, '') for x in origpack.iterobjects()],
+                   len(origpack))
+        return Pack(basename)
+
+    def test_keep_no_message(self):
+        p = self.get_pack(pack1_sha)
+        p = self._copy_pack(p)
+
+        keepfile_name = p.keep()
+        # file should exist
+        self.assertTrue(os.path.exists(keepfile_name))
+
+        f = open(keepfile_name, 'r')
+        try:
+            buf = f.read()
+            self.assertEqual('', buf)
+        finally:
+            f.close()
+
+    def test_keep_message(self):
+        p = self.get_pack(pack1_sha)
+        p = self._copy_pack(p)
+
+        msg = 'some message'
+        keepfile_name = p.keep(msg)
+
+        # file should exist
+        self.assertTrue(os.path.exists(keepfile_name))
+
+        # and contain the right message, with a linefeed
+        f = open(keepfile_name, 'r')
+        try:
+            buf = f.read()
+            self.assertEqual(msg + '\n', buf)
+        finally:
+            f.close()
+
     def test_name(self):
         p = self.get_pack(pack1_sha)
         self.assertEquals(pack1_sha, p.name())
